/**
 * 
 */
package org.processmining.models.operationalsupport;

import java.util.ArrayList;

import org.processmining.models.operationalsupport.xml.OSXMLConverter;

/**
 * Data structure for storing a result for requesting recommendation for
 * scheduling in a process management system. (based on an array list of
 * ProviderResponse objects)
 * 
 * @author Christian W. Guenther (christian@deckfour.org)
 * 
 */
public class Response {

	/**
	 * 
	 */
	private static final long serialVersionUID = 5792568069434244616L;
	protected String requestId;
	private final ArrayList<ProviderResponse> providers;

	/**
	 * Creates a new, empty recommendation result for the identified request
	 * 
	 * @param requestId
	 *            the unique ID of the request to which this is a result.
	 */
	public Response(String requestId) {
		super();
		this.requestId = requestId;
		providers = new ArrayList<ProviderResponse>();
	}

	/**
	 * @return the unique ID of the request to which this is a result
	 */
	public String getrequestId() {
		return requestId;
	}

	/*
	 * TODO: add priorities here by adding recommendations (non-Javadoc)
	 * 
	 * @see java.util.ArrayList#add(java.lang.Object)
	 */
	public boolean add(ProviderResponse rec) {
		return providers.add(rec);
	}

	public String toString() {
		try {
			return (new OSXMLConverter()).responseToXML(this);
		} catch (Exception e) {
			e.printStackTrace();
			return "ERROR";
		}
	}

	public Iterable<ProviderResponse> getProviderResponses() {
		return providers;
	}

	public int getProvidersCount() {
		return providers.size();
	}

	/*
	 * public void print(){
	 * System.out.println("Printing Response for request "+this.requestId); for
	 * (ProviderResponse pr: providers){ pr.print(); } }
	 */
}
