package org.processmining.models.operationalsupport.net.provider;

import java.io.IOException;

import org.processmining.framework.util.socket.ServiceEnvironment;
import org.processmining.models.operationalsupport.ProviderResponse;
import org.processmining.models.operationalsupport.Request;
import org.processmining.models.operationalsupport.net.service.OSService;

public interface Provider {

	/**
	 * Returns the service that owns this provider (a provider can be owned by
	 * one service only);
	 * 
	 * @return
	 */
	public OSService getOwner();

	/**
	 * <p>
	 * This is the main request method, which can be used to retrieve
	 * recommendations for specific queries. All communication and encoding of
	 * parameters can be assumed to be performed in subclasses implementing this
	 * interface.
	 * </p>
	 * <p>
	 * <b>Note:</b> This method may block for an unspecified time, including
	 * network transport and resolving the request on the provider side. Use
	 * this method in a separate thread if you wish your application to remain
	 * responsive.<br>
	 * Also note that the request can fail, both due to transmission errors
	 * (which will yield an <code>IOException</code>, and due to the provider's
	 * inability to resolve the request (i.e., an error occurred while handling
	 * the issued request, signaled by an ordinary <code>Exception</code> being
	 * thrown; or, the request did not yield any usable result, which results in
	 * a <code>null</code> response value.
	 * </p>
	 * 
	 * @param environment
	 *            The context which is monitoring the service
	 * @param request
	 *            The request for recommendations.
	 * @return The resulting set of recommendations (correlated with the issued
	 *         request), or <code>null</code>, of no result could be obtained
	 *         (may also be empty result list).
	 * @throws Exception
	 *             Signals an error in the recommendation provider
	 *             implementation.
	 * @throws IOException
	 *             Signals an error due to network transport problems.
	 */
	public void populateResponse(ServiceEnvironment environment, Request request, ProviderResponse response);

}