/**
 * 
 */
package org.processmining.models.operationalsupport.xml;

import org.deckfour.xes.xstream.XConverter;
import org.processmining.models.operationalsupport.Analysis;
import org.processmining.models.operationalsupport.Prediction;
import org.processmining.models.operationalsupport.ProviderResponse;
import org.processmining.models.operationalsupport.Recommendation;

import com.thoughtworks.xstream.XStream;
import com.thoughtworks.xstream.converters.MarshallingContext;
import com.thoughtworks.xstream.converters.UnmarshallingContext;
import com.thoughtworks.xstream.io.HierarchicalStreamReader;
import com.thoughtworks.xstream.io.HierarchicalStreamWriter;

/**
 * @author mpesic
 * 
 */
class XOSProviderResponseConverter extends XConverter {
	private static final String PROVIDER_NAME = "name";
	private static final String ANALYSIS = "analysis";
	private static final String PREDICTION = "prediction";
	private static final String RECOMMENDATION = "recommendation";
	private static final String ITEM = "item";

	public void marshal(Object value, HierarchicalStreamWriter writer, MarshallingContext context) {
		ProviderResponse providerResponse = (ProviderResponse) value;

		writer.addAttribute(PROVIDER_NAME, providerResponse.getProviderName()); // set the provider name

		writer.startNode(ANALYSIS); // add analysis
		for (Analysis a : providerResponse.getAnalysis()) {
			writer.startNode(ITEM);
			context.convertAnother(a);
			writer.endNode();
		}
		writer.endNode();

		writer.startNode(PREDICTION); // add prediction
		for (Prediction p : providerResponse.getPrediction()) {
			writer.startNode(ITEM);
			context.convertAnother(p);
			writer.endNode();
		}
		writer.endNode();

		writer.startNode(RECOMMENDATION); // add recommendation
		for (Recommendation r : providerResponse.getRecommendation()) {
			writer.startNode(ITEM);
			context.convertAnother(r);
			writer.endNode();
		}
		writer.endNode();
	}

	public Object unmarshal(HierarchicalStreamReader reader, UnmarshallingContext context) {
		String providerName = reader.getAttribute(PROVIDER_NAME);
		ProviderResponse provioderResponse = new ProviderResponse(providerName);

		while (reader.hasMoreChildren()) {
			reader.moveDown();
			if (ANALYSIS.equals(reader.getNodeName())) { // get analysis items
				while (reader.hasMoreChildren()) {
					reader.moveDown();
					Analysis a = (Analysis) context.convertAnother(provioderResponse, Analysis.class);
					provioderResponse.addAnalysis(a);
					reader.moveUp();
				}
			}
			if (PREDICTION.equals(reader.getNodeName())) { // get prediction items
				while (reader.hasMoreChildren()) {
					reader.moveDown();
					Prediction p = (Prediction) context.convertAnother(provioderResponse, Prediction.class);
					provioderResponse.addPrediction(p);
					reader.moveUp();
				}
			}
			if (RECOMMENDATION.equals(reader.getNodeName())) { // get recommendation items
				while (reader.hasMoreChildren()) {
					reader.moveDown();
					Recommendation r = (Recommendation) context.convertAnother(provioderResponse, Recommendation.class);
					provioderResponse.addRecommendation(r);
					reader.moveUp();
				}
			}
			reader.moveUp();
		}

		return provioderResponse;
	}

	@SuppressWarnings("unchecked")
	public boolean canConvert(Class clazz) {
		return clazz.equals(ProviderResponse.class);
	}

	public void registerAliases(XStream stream) {
		super.registerAliases(stream);
		stream.alias("provider", ProviderResponse.class);
		stream.alias("analysis", Analysis.class);
		stream.alias("prediiction", Prediction.class);
		stream.alias("recommendation", Recommendation.class);
		stream.alias("measure", String.class);
	}
}
